#include <iostream>
#include <vector>
#include <regex>
#include <sstream>
using namespace std;

/*Game of Life prototype
Author: William Matheson
Course: INFT2100 Section 702
November-December 2012*/

bool lifeGrid[20][60]; //20 rows, 60 cols
//each space represents a cell

//displays grid
void displayGrid()
{
	for (int i=0; i<20; i++)
	{
		for (int j=0; j<60; j++)
		{
			if (lifeGrid[i][j])//alive
				cout << "X";
			else//dead
				cout << " ";
		}
		cout << endl; //new line for next row
	}
}

//vectorizes a string by delimiter
//http://stackoverflow.com/a/237280
vector<string> &split(const string &s, char delim, vector<string> &elems) {
    stringstream ss(s);
    string item;
    while(getline(ss, item, delim)) {
        elems.push_back(item);
    }
    return elems;
}

//takes a string and a delimiter and calls the function to vectorize
//the string by delimiter, returns the split string
//http://stackoverflow.com/a/237280
vector<string> split(const string &s, char delim) {
    vector<string> elems;
    return split(s, delim, elems);
}

//for making integers of strings
int string2Int(string numberIn)
{
	int number;
	stringstream stream;
	stream << numberIn;
	stream >> number;
	return number;
}

int main(){
	cout << "Welcome to the Game of Life, version 1.0." << endl;
	cout << "Based on a concept by John Horton Conway." << endl;
	cout << "Coded by William Matheson." << endl << endl;

	//all cells initially dead
	for (int i=0; i<20; i++)
		for (int j=0; j<60; j++)
			lifeGrid[i][j] = false;

	//store coordinates to make dead
	vector<int> makeDeadRow;
	vector<int> makeDeadCol;
	//store coordinates to make alive
	vector<int> makeAliveRow;
	vector<int> makeAliveCol;

	int livingNeighbours;
	int generationNum = 1;

	int rowNum = -1;//between 0 and 19
	int colNum = -1;//between 0 and 59
	//these will be one less than what the user enters
	
	bool inputValid = false;
	string input = "";
	vector<string> coords;
	string quitLine = "";
	
	//if user input isn't in form (-)n,(-)n (max 2 digits each)
	//generate an error
	regex inputFormat("^-?[0-9]{1,2},-?[0-9]{1,2}$");
	
	//user is asked which cells are alive
	//have to take the user's string, split it on a comma and process the numbers
	//also match it against a regexp to protect from shenanigans
	while(rowNum != -2 && colNum != -2)
	{//get living cells
		while(!inputValid)
		{//try to get this coordinate!
			cout << "Enter coordinate (row, column), or -1,-1 to stop: ";
			getline(cin,input);
			if(cin.eof()) return -1; // if Ctrl+Z is entered
			if (regex_match(input,inputFormat))
			{
				coords = split(input,',');
				rowNum = string2Int(coords.at(0)) - 1;
				colNum = string2Int(coords.at(1)) - 1;
				if ((rowNum >= 0 && rowNum <= 19) && (colNum >= 0 && colNum <= 59))
				{
					if (lifeGrid[rowNum][colNum] == false)
					{
						lifeGrid[rowNum][colNum] = true;
						inputValid = true;
					}
					else cout << "You already made cell " << rowNum + 1 << "," << colNum + 1 << " alive." << endl;
				}
				else if (rowNum == -2 && colNum == -2)
					inputValid = true;
				else
					cout << "Coordinate out of range!" << endl;
			}
			else
				cout << "Invalid coordinate format." << endl;
		}
		inputValid = false;
	}
	cout << "Okay, let's start the game!" << endl << endl;

	cout << "This is the initial configuration you specified:" << endl;
	//Display the grid
	displayGrid();
	cout << "Press Enter to continue.";
	cin.get();
	if (cin.eof()) return -1; // if Ctrl+Z is entered

	while (quitLine.compare("n") !=0 && quitLine.compare("N") !=0)
	{//keep displaying generations

	    //count each cell's living neighbours
		for (int i=0; i<20; i++)
		{
			for (int j=0; j<60; j++)
			{
				livingNeighbours = 0;
				if (i>0 && j>0)
					if (lifeGrid[i-1][j-1])//NW
					    livingNeighbours ++;
				if (i>0)
					if (lifeGrid[i-1][j])//N
						livingNeighbours ++;
				if (i>0 && j<59)
					if (lifeGrid[i-1][j+1])//NE
						livingNeighbours ++;
				if (j>0)
					if (lifeGrid[i][j-1])//W
						livingNeighbours ++;
				if (j<59)
					if (lifeGrid[i][j+1])//E
						livingNeighbours ++;
				if (i<19 && j>0)
					if (lifeGrid[i+1][j-1])//SW
						livingNeighbours ++;
				if (i<19)
					if (lifeGrid[i+1][j])//S
						livingNeighbours ++;
				if (i<19 && j<59)
					if (lifeGrid[i+1][j+1])//SE
						livingNeighbours ++;

				//if this cell is alive, it can stay alive if it has 2 or 3 neighbours
				if (lifeGrid[i][j])
				    if ((livingNeighbours < 2) || (livingNeighbours > 3))
					{
						makeDeadRow.push_back(i);
						makeDeadCol.push_back(j);
					}

				//if this cell is dead, it stays dead unless it has 3 neighbours
				else
				    if (livingNeighbours == 3)
					{
						makeAliveRow.push_back(i);
						makeAliveCol.push_back(j);
					}
			}
		}

		//actually make cells dead
		for (size_t k=0;k<makeDeadRow.size();k++)
			lifeGrid[makeDeadRow.at(k)][makeDeadCol.at(k)] = false;
		
	    //actually make cells alive
		for (size_t m=0;m<makeAliveRow.size();m++)
			lifeGrid[makeAliveRow.at(m)][makeAliveCol.at(m)] = true;
	
		//clear the coordinate containers
		makeDeadRow.clear();
		makeDeadCol.clear();
		makeAliveRow.clear();
		makeAliveCol.clear();

		//display grid
		if (generationNum == 1)
			cout << "These are the results after " << generationNum << " generation:" << endl;
		else
			cout << "These are the results after " << generationNum << " generations:" << endl;
		displayGrid();

		generationNum ++;

		//want to run another generation?
		cout << "Continue? (Y/N): ";
			getline(cin,quitLine);
			if(cin.eof()) return -1; // if Ctrl+Z is entered
	}

	cout << "Goodbye! See you next time!" << endl;
	cin.get();
    return 0;
}